@@ -504,7 +504,8 @@ else  {Wsq<-as.matrix("NA")}
           ll <- c(ll,paste("link",i,sep=""))
        } 
        dimnames(beta) <- list(lbeta,"BETA")  
-       dimnames(covbeta) <- list(lbeta,lbeta)
+       if(!is.null(colnames(X))){dimnames(beta) <- list(colnames(X),"BETA")}
+      dimnames(covbeta) <- list(lbeta,lbeta)
        dimnames(Lobs) <- list(ll,"OBS LINK")   
        dimnames(L) <- list(ll,"ML LINK")  
        dimnames(covL) <- list(ll,ll) 
@@ -570,221 +571,13 @@ resid=resid,presid=presid,adjresid=adjresid,covresid=covresid,
 Gsq=Gsq,Xsq=Xsq,Wsq=Wsq,df=lenh,
 beta=beta,covbeta=covbeta, Lobs=Lobs, L=L,covL=covL,Lresid=Lresid,
 iter=iter, norm.diff=norm.diff,norm.score=norm.score,
-h.fct=h.fct,derht.fct=derht.fct,L.fct=L.fct,derLt.fct=derLt.fct,X=X,
-U=U,Z=Z,ZF=ZF,Zlist=Zlist,version=version)
+h.fct=h.fct,derht.fct=derht.fct,L.fct=L.fct,derLt.fct=derLt.fct,
+d.fct=d.fct,derdt.fct=derdt.fct,
+X=X,U=U,Z=Z,ZF=ZF,Zlist=Zlist,version=version)
 class(modlist)="mphfit"
 modlist
 }
-######################   end mph.fit   ########################################## 
-summary.hmmmfit<-function(object,cell.stats=TRUE,...){mph.summary(object,cell.stats=cell.stats,model.info=FALSE)}
-summary.mphfit<-function(object,...){mph.summary(object,cell.stats=TRUE,model.info=TRUE)}
-print.mphfit<-function(x,...){mph.summary(x,cell.stats=FALSE,model.info=FALSE)}
-
-######################   Begin mph.summary ######################################  
-mph.summary <- function(mph.out,cell.stats=FALSE,model.info=FALSE) {
-#
-#  This function is used in conjunction with the ML fitting 
-#  function `mph.fit'.  It computes and prints a collection 
-#  of summary statistics of the fitted MPH model given in `mph.out',
-#  which is the result of `mph.fit'.  That is, 
-#  mph.out <- mph.fit(y,Z,ZF,...)
-#  
-#  Author:  Joseph B. Lang, 
-#           Dept of Statistics 
-#              and Actuarial Science
-#           Univ of Iowa, Iowa City, IA 52242
-#           8/16/01
-#  Last Updated:  3/30/04
-#
-#  INPUT 
-#     Required:
-#          mph.out = result of `mph.fit'
-#
-#     Optional:
-#          cell.stats = logical variable indicating whether cell
-#                       specific statistics are to be output
-#                       (default: cell.stats=FALSE)?
-#          model.info = logical variable indicating whether model
-#                       information is to be output
-#                       (default: model.info=FALSE)
-#                    
-# 
-a <- mph.out 
-if(class(a)=="hmmmfit"){
-a$df=a$df+dim(a$Zlist$DMAT)[1]-dim(a$Zlist$DMAT)[2]-a$model$modello$strata}
-cat("\nOVERALL GOODNESS OF FIT:")
-cat("\n")
-cat("    Likelihood Ratio Stat (df=",a$df,"):  Gsq = ",
-    round(a$Gsq,5))
-if (a$df > 0) cat(" (p = ",signif(1-pchisq(a$Gsq,a$df),5),")")
-cat("\n")
-cat("    Pearson's Score Stat  (df=",a$df,"):  Xsq = ",
-    round(a$Xsq,5))
-if (a$df > 0) cat(" (p = ",signif(1-pchisq(a$Xsq,a$df),5),")")
-cat("\n")
-#if(a$Wsq != "NA"){
-#cat("    Generalized Wald Stat (df=",a$df,"):  Wsq = ",
-#    round(a$Wsq,5))
-#if (a$df > 0) cat(" (p = ",signif(1-pchisq(a$Wsq,a$df),5),")")}
-cat("\n")
-sm <- 100*length(a$m[a$m < 5])/length(a$m)  
-if ((sm > 75)&(a$df > 0)) {
-    cat("\n    WARNING:", paste(sm,"%",sep=""),
-    "of expected counts are less than 5. \n")
-    cat("             Chi-square approximation may be questionable.")
-}
-cat("\n") 
-if (cell.stats==TRUE) {
-
-if (a$L[1] != "NA") {
-  cat("\nLINEAR PREDICTOR MODEL RESULTS...")
-  cat("\n")
-  sbeta <- as.matrix(sqrt(abs(diag(a$covbeta))))
-  z <- a$beta/sbeta
-  pval <- 2*(1-pnorm(abs(z)))
-  dimnames(sbeta)[2] <- "StdErr(BETA)"
-  dimnames(z)[2] <- "Z-ratio"
-  dimnames(pval)[2] <- "p-value"
- # betanames<-colnames(a$matrici$X)
-
-if(class(a)=="mphfit"||a$model$modello$strata >1){
-  print(cbind(a$beta,sbeta,z,pval))
-
-  cat("\n")}
-  stdL <- as.matrix(sqrt(diag(a$covL)))
-  dimnames(stdL)[2] <- "StdErr(L)"
-  LLL<-round(cbind(a$Lobs,a$L,stdL,a$Lresid),4)
-if(class(a)=="hmmmfit"){
-if(is.null(a$model$names)){
-descr<-hmmm.model.summary(a$model,printflag=FALSE)
-descr<-rep(descr[,1],descr[,4])}
-else{
-descr<-hmmm.model.summary(a$model,printflag=FALSE)
-
-descr<-hmmm.model.summary(a$model,printflag=FALSE)
-descr<-rep(descr[,2],descr[,6])}
-
- # descr<-hmmm.model.summary(modello,print=FALSE)
-  #intnames<-paste("int",descr[,1],sep="_")
-  intnames<-descr
-  rownames(LLL)<-rep(intnames,dim(a$model$matrici$Z)[2])}
-  print(LLL)
-  cat("\n")
-}
-############################if (cell.stats==TRUE) {
-   stdm <- as.matrix(sqrt(diag(a$covm)))
-   stdp <- as.matrix(sqrt(diag(a$covp)))
-   dimnames(stdm)[2] <- "StdErr(FV)"
-   dimnames(stdp)[2] <- "StdErr(PROB)"
-   cat("\nCELL-SPECIFIC STATISTICS...")
-   cat("\n")
-   print(round(cbind(a$y,a$m,stdm,a$p,stdp,a$adjresid),5))
-   cat("\n")
-}
-cat("\nCONVERGENCE STATISTICS...")
-cat("\n")
-cat("    iterations =",a$iter)
-cat("\n")
-cat("    norm.diff  =",signif(a$norm.diff,6))
-cat("\n")
-cat("    norm.score =",signif(a$norm.score,6))
-cat("\n")
 
 
-if (model.info==TRUE) {
-cat("\nMODEL INFORMATION...")
-cat("\n")
-  if (a$L[1] != "NA") {
-    cat("Linear Predictor Model Link Function  L.fct:\n")
-    print(a$L.fct)
-    cat("\n")
-    cat("Derivative of Transpose Link Function derLt.fct: \n")
-    print(a$derLt.fct)
-    cat("\n")
-    cat("Linear Predictor Model Design Matrix  X: \n")
-    print(a$X)
-    cat("\n")
-    cat("U = Orthogonal Complement of X: \n")
-    if (is.matrix(a$U)) print(a$U)
-    else cat("\n  ",a$U,"\n")
-  }
-  cat("\n")  
-  cat("Constraint Function  h.fct:\n")
-  print(a$h.fct)
-  cat("\n")
-  cat("Derivative of Transpose Constraint Function derht.fct:\n")
-  print(a$derht.fct)
-  cat("\n")
-  cat("Population Matrix Z: \n")
-  print(a$Z)
-  cat("\n")
-  cat("Sampling Constraint Matrix ZF:\n")
-  print(a$ZF)
-  cat("\n")
-}
-cat("\nFITTING PROGRAM USED: ", a$version,"\n\n")
-}
-######################   end mph.summary     ######################################
 
-######################   begin num.deriv.fct ######################################       
-num.deriv.fct <- function(f.fct,m) {
-# 
-#   Author: Joseph B. Lang, Univ of Iowa
-#   Created:  c. 8/25/00 (last update: 3/30/04)
-#
-#   The numerical derivative of the transpose of function f.fct is 
-#   computed at the value m.  If f.fct is a mapping from 
-#   Rp to Rq then the result is a pxq matrix.
-#     I.e. Result is approximation to 
-#         d f.fct(m)^T/d m.
-# 
-  eps <- (.Machine$double.eps)^(1/3)
-  d <- eps * m + eps  
-  lenm <- length(m)
-  E <- diag(c(d)) 
-  f1 <- f.fct(m+E[,1])
-  lenf <- length(f1)
-  Ft <- (f1-f.fct(m-E[,1]))/(2*d[1])
-  for (j in 2:lenm) {
-     Ft <- cbind(Ft,((f.fct(m+E[,j])-f.fct(m-E[,j]))/(2*d[j])))
-  }
-  dimnames(Ft) <- NULL
-  t(Ft)
-}
-######################   end num.deriv.fct ######################################
-   
-
-######################   begin create.U    ###################################### 
-create.U <- function(X) {
-#
-#   Author: Joseph B. Lang, Univ of Iowa
-#   Created:  8/19/01  (last update: 3/30/04)
-#
-# This program creates a full-column rank matrix, U, with column space 
-# equal to the orthogonal complement of the column space of X.  That is,
-# U has column space equal to the null space of X^T.
-#
-#  Input:  X must be of full column rank
-#  
-  nrowX <- nrow(X)
-  u <- nrowX - ncol(X)
-  if (u == 0) {U <- 0}
-  else {w.mat <- matrix(runif(nrowX*u,1,10),nrowX,u)
-    U <- w.mat - X%*%solve(t(X)%*%X)%*%t(X)%*%w.mat
-  }
-  U
-}
-######################   end create.U     ######################################
- getnames<-function(dat,st=3,sep=" "){
-#dat=aggregated data frame with frequencies in the last column
-#st length of the string for every category name
-#sep separetor of category names of a cell
-fa<-dim(dat)[2]-1
-a<-substr(dat[[1]],1,st)
-for(i in 2:fa){
-a<-paste(a,substr(dat[[i]],1,st),sep=sep)
-}
-y<-as.matrix(dat[[fa+1]])
-rownames(y)<-a
-y}
 
